@@ -1,7 +1,5 @@
 require "mkmf"
 
-have_header("dummy.h")
-
 dir_config('narray',$sitearchdir,$sitearchdir)
 dir_config('netcdf','/usr/local')
 
@@ -25,7 +23,7 @@ print <<EOS
    in /netcdfdir/include and the library in /netcdfdir/lib/),
    try the following:
 
-   % ruby extconf.rb --with-netcdf=/netcdfdir
+   % ruby extconf.rb --with-netcdf-dir=/netcdfdir
 
    Alternatively, you can specify the two directory separately
    with --with-netcdf-include and --with-netcdf-lib.
@@ -33,8 +31,11 @@ EOS
    exit(-1)
 end
 
-create_makefile "numru/netcdfraw"
+if defined?(__CYGWIN__) && VERSION < '1.7'
+   have_library("narray") || raise("ERROR: narray brary is not found")
+end
 
+create_makefile "numru/netcdfraw"
 
 ######  Modify Makefile: #######
 File.rename("Makefile","Makefile.orig")
@@ -42,18 +43,16 @@ oldmkfl = File.open("Makefile.orig")
 newmkfl = File.open("Makefile","w")
 oldmkfl.each_line{ |line|
    case(line)
-   when /^install:/
-      newmkfl.puts("install: site-install")    # always site-install
    when /^distclean:/
       newmkfl.puts(line)
-      newmkfl.puts("\t\t@$(RM) *.nc demo/*.nc demo/*~ lib*~ doc/*~ test/*.nc test/*~ Makefile.orig")
+      newmkfl.puts("\t\t@$(RM) *.nc demo/*.nc demo/*~ lib/*~ doc/*~ test/*.nc test/*~ Makefile.orig")
    when /^all:/
       newmkfl.puts(line)
       newmkfl.puts("")         
       newmkfl.puts("test: all")            # insert the "test" target
-      newmkfl.puts("\t\t@cd test && ruby test.rb && echo 'test did not fail :-p' && cd ..") 
+      newmkfl.puts("\t\t@cd test && ruby test.rb && echo 'test did not fail :-p (please ignore the warnings)' && cd ..") 
    when /lib\/netcdf/
-      line = line.chomp! + "/netcdf.rb"
+      line = line.chomp! + "/"
       newmkfl.puts(line)
    else
       newmkfl.puts(line)
