@@ -5,6 +5,8 @@ require 'numru/netcdfraw'
 module NumRu
   class NetCDF
 
+    Max_Try = 100
+
     def NetCDF.open(filename,mode="r",share=false)
        call_create=false   # false-> nc_open; true->nc_create
        case(mode)
@@ -14,7 +16,7 @@ module NumRu
           call_create=true
 	  mode=NC_CLOBBER
        when "a","a+","r+","ab","a+b","r+b"    # append if exits
-	  if( File.exists? )
+	  if( File.exists?(filename) )
 	     mode=NC_WRITE
 	  else
 	     call_create=true   #(nonexsitent --> create)
@@ -35,7 +37,7 @@ module NumRu
        if(!call_create)
 	  nc_open(filename,omode)
        else
-	  nc_create(filename.omode)
+	  nc_create(filename,omode)
        end
     end
     
@@ -66,12 +68,74 @@ module NumRu
       nc_create(filename,cmode)
     end
     
+    class << NetCDF
+       def clean_tmpfile(path)
+	  lambda{
+	     print "removing ", path, "..." if $DEBUG
+	     if File.exist?(path)
+		File.unlink(path) 
+	     end
+	     print "done\n" if $DEBUG
+	  }
+       end
+       protected :clean_tmpfile
+    end
+
+    def NetCDF.create_tmp(tmpdir=ENV['TMPDIR']||ENV['TMP']||ENV['TEMP']||'.', 
+			  share=false)
+       basename = 'temp'
+       if $SAFE > 0 and tmpdir.tainted?
+	  tmpdir = '.'
+       end
+
+       n = 0
+       while true
+	 begin
+	   tmpname = sprintf('%s/%s%d_%d.nc', tmpdir, basename, $$, n)
+	   unless File.exist?(tmpname)
+	      netcdf = NetCDF.create(tmpname, true, share)
+	      ObjectSpace.define_finalizer(netcdf, 
+					   NetCDF.clean_tmpfile(tmpname))
+	      break
+	   end
+	 rescue
+	   raise "cannot generate tempfile `%s'" % tmpname if n >= Max_Try
+	 end
+	 n += 1
+       end
+       netcdf
+    end
+
+
     def put_att(attname,val,atttype=nil)
        put_attraw(attname,val,atttype)
     end
     
-    
-    #$B%$%F%l!<%?4X78(B	 
+    def def_var_with_dim(name, vartype, shape_ul0, dimnames)
+       # Same as def_var but defines dimensions first if needed.
+       # Use zero in shape to define an unlimited dimension.
+       if (shape_ul0.length != dimnames.length ) then
+	  raise 'lengths of shape and dimnames do not agree'
+       end
+       dims = []
+       dimnames.each_index{ |i|
+	  dim = self.dim( dimnames[i] )
+	  if ( dim != nil ) then
+	     # dim exists --> check the length
+	     if (shape_ul0[i] != dim.length_ul0 ) then
+		raise "dimension length do not agree: #{i}th dim: "+\
+		"#{shape_ul0[i]} and #{dim.length_ul0}"
+	     end
+	     dims.push(dim)
+	  else
+	     # dim does not exist --> define it
+	     dims.push( def_dim( dimnames[i], shape_ul0[i] ) )
+	  end
+       }
+       def_var(name, vartype, dims)
+    end
+
+    # Iterators:
     def each_dim
       num_dim=ndims()    
       for dimid in 0..num_dim-1
@@ -96,6 +160,26 @@ module NumRu
       end
     end
     
+    def dims( names=nil )   # return all if names==nil
+       if names == nil
+	  dims = (0..ndims()-1).collect{|dimid| id2dim(dimid)}
+       else
+	  raise "names is not an array" if ! names.is_a?(Array)
+	  dims = names.collect{|name| dim(name)}
+       end
+       dims
+    end
+
+    def vars( names=nil )   # return all if names==nil
+       if names == nil
+	  vars = (0..nvars()-1).collect{ |varid| id2var(varid) }
+       else
+	  raise "names is not an array" if ! names.is_a?(Array)
+	  vars = names.collect{|name| var(name)}
+       end
+       vars
+    end
+
     def dim_names
       num_dim=ndims()
       names=[]
@@ -103,7 +187,7 @@ module NumRu
 	obj_Dim=id2dim(dimid)    
 	names=names+[obj_Dim.name]
       end
-      return names	
+      return names
     end    
     
     def var_names
@@ -126,10 +210,8 @@ module NumRu
       return names
     end
     
-    alias :uri :path
-    
     def inspect
-      "NetCDF:"+uri
+      "NetCDF:"+path
     end
     
   end
@@ -138,18 +220,20 @@ module NumRu
   class NetCDFVar
     
     class << NetCDFVar
-       def NetCDFVar.new(uri,mode="r",share=false)
-	  raise "uri must be a String" if !uri.is_a?(String)
-	  filename,varname = uri.split('?')
-	  raise 'uri must be "filename?varname"' if filename==nil
-	  raise 'uri must be "filename?varname"' if varname==nil
-	  file = NetCDF.open(filename,mode,share)
+       def new(file,varname,mode="r",share=false)
+	  if(file.is_a?(String))
+	     file = NetCDF.open(file,mode,share)
+	  elsif(!file.is_a?(NetCDF))
+	    raise "1st arg must be a NetCDF (file object) or a String (path)"
+	  end
 	  file.var(varname)
        end
 
        alias open new
     end
 
+    alias :rank :ndims
+
     def each_att
       num_att=natts()
       for attnum in 0..num_att-1
@@ -177,12 +261,106 @@ module NumRu
    def put_att(attname,val,atttype=nil)
       put_attraw(attname,val,atttype)
    end
+
+   def shape_ul0
+      sh = []
+      dims.each{|d|
+	 if d.unlimited? then
+	    sh.push(0)
+	 else
+	    sh.push(d.length)
+	 end
+      }
+      sh
+   end
+   
+   def shape_current
+      sh = []
+      dims.each{|d|
+	 sh.push(d.length)
+      }
+      sh
+   end
    
    # The put and get methods in the NetCDFVar class
    
-   def put(var,hash=nil)
+   def scaled_put(var,hash=nil)
+      sf = att('scale_factor')
+      ao = att('add_offset')
+      if ( sf == nil && ao == nil ) then
+	 # no scaling --> just call put
+	 simple_put(var,hash)
+      else
+	 if (sf != nil) 
+	    csf = sf.get
+	    if csf.is_a?(NArray) then  # --> should be a numeric
+	       csf = csf[0]
+	    elsif csf.is_a?(String)
+	       raise "scale_factor is not a numeric"
+	    end
+	    if(csf == 0) then; raise "zero scale_factor"; end
+	 else
+	    csf = 1.0      # assume 1 if not defined
+	 end
+	 if (ao != nil) 
+	    cao = ao.get
+	    if cao.is_a?(NArray) then  # --> should be a numeric
+	       cao = cao[0]
+	    elsif csf.is_a?(String)
+	       raise "add_offset is not a numeric"
+	    end
+	 else
+	    cao = 0.0      # assume 0 if not defined
+	 end
+	 simple_put( (var-cao)/csf, hash )
+      end
+   end
+
+   def scaled_get(hash=nil)
+      sf = att('scale_factor')
+      ao = att('add_offset')
+      if ( sf == nil && ao == nil ) then
+	 # no scaling --> just call get
+	 simple_get(hash)
+      else
+	 if (sf != nil) 
+	    csf = sf.get
+	    if csf.is_a?(NArray) then  # --> should be a numeric
+	       csf = csf[0]
+	    elsif csf.is_a?(String)
+	       raise "scale_factor is not a numeric"
+	    end
+	    if(csf == 0) then; raise "zero scale_factor"; end
+	 else
+	    csf = 1.0      # assume 1 if not defined
+	 end
+	 if (ao != nil) 
+	    cao = ao.get
+	    if cao.is_a?(NArray) then  # --> should be a numeric
+	       cao = cao[0]
+	    elsif csf.is_a?(String)
+	       raise "add_offset is not a numeric"
+	    end
+	 else
+	    cao = 0.0      # assume 0 if not defined
+	 end
+	 var = simple_get(hash)
+	 case (vartype)
+	 when "int"
+	    var = var.to_type( NArray::FLOAT )
+	 when "byte","sint","char"
+	    var = var.to_type( NArray::SFLOAT )
+	 else
+	 end
+	 csf*var+cao
+      end
+   end
+
+   def simple_put(var,hash=nil)
      if hash==nil
-       if self.vartype == "byte"
+       if self.vartype == "char"
+	 put_var_char(var)
+       elsif self.vartype == "byte"
 	 put_var_byte(var)
        elsif self.vartype == "sint"
 	 put_var_sint(var)
@@ -196,7 +374,9 @@ module NumRu
 	 raise NetcdfError,"variable type isn't supported in netCDF" 
        end
      elsif hash.key?("index")==true 
-       if self.vartype=="byte"
+       if self.vartype == "char"
+	 put_var1_char(var,hash["index"])
+       elsif self.vartype=="byte"
 	 put_var1_byte(var,hash["index"])
        elsif self.vartype=="sint"
 	 put_var1_sint(var,hash["index"])
@@ -211,7 +391,9 @@ module NumRu
        end
      elsif hash.key?("start")==true
        if hash.key?("end")==false && hash.key?("stride")==false
-	 if self.vartype=="byte"
+	 if self.vartype == "char"
+	   put_vars_char(var,hash["start"],nil,nil)
+	 elsif self.vartype=="byte"
 	   put_vars_byte(var,hash["start"],nil,nil)
 	 elsif self.vartype=="sint"
 	   put_vars_sint(var,hash["start"],nil,nil)
@@ -225,7 +407,9 @@ module NumRu
 	   raise NetcdfError, "variable type isn't supported in netCDF"
 	 end
        elsif hash.key?("end")==true && hash.key?("stride") == false
-	 if self.vartype=="byte"
+	 if self.vartype == "char"
+	   put_vars_char(var,hash["start"],hash["end"],nil)
+	 elsif self.vartype=="byte"
 	   put_vars_byte(var,hash["start"],hash["end"],nil)
 	 elsif self.vartype=="sint"
 	   put_vars_sint(var,hash["start"],hash["end"],nil)
@@ -239,7 +423,9 @@ module NumRu
 	   raise NetcdfError, "variable type isn't supported in netCDF"
 	 end
        elsif hash.key?("end")==false && hash.key?("stride")==true
-	 if self.vartype=="byte"
+	 if self.vartype == "char"
+	   put_vars_char(var,hash["start"],nil,hash["stride"])
+	 elsif self.vartype=="byte"
 	   put_vars_byte(var,hash["start"],nil,hash["stride"])
 	 elsif self.vartype=="sint"
 	   put_vars_sint(var,hash["start"],nil,hash["stride"])
@@ -253,7 +439,9 @@ module NumRu
 	   raise NetcdfError, "variable type isn't supported in netCDF"
 	 end
        else hash.key?("end")==true && hash.key?("stride")==true
-	 if self.vartype=="byte"
+	 if self.vartype == "char"
+	   put_vars_char(var,hash["start"],hash["end"],hash["stride"])
+	 elsif self.vartype=="byte"
 	   put_vars_byte(var,hash["start"],hash["end"],hash["stride"])
 	 elsif self.vartype=="sint"
 	   put_vars_sint(var,hash["start"],hash["end"],hash["stride"])
@@ -271,14 +459,18 @@ module NumRu
        raise NetcdfError,"{'start'}=>[ARRAY] or {'index'}=>[ARRAY] is needed"
      end
    end
-   
-   def get(hash=nil)
+
+   alias put simple_put
+
+   def simple_get(hash=nil)
      t_var = self.vartype
      if hash == nil
-       if t_var == "byte"
+       if t_var == "char"
+	 get_var_char
+       elsif t_var == "byte"
 	 get_var_byte
        elsif t_var == "sint"
-	 get_var_byte
+	 get_var_sint
        elsif t_var == "int"
 	 get_var_int
        elsif t_var == "sfloat"
@@ -286,11 +478,13 @@ module NumRu
        elsif t_var == "float"
 	 get_var_float
        else
-	 raise NetcdfError, "variable type isn't supported in netCDF"
+	  raise NetcdfError, "variable type #{t_var} isn't supported in netCDF"
        end
      elsif hash.key?("index")==true
        ind = hash["index"]
-       if t_var == "byte"
+       if t_var == "char"
+	 get_var1_char(ind)
+       elsif t_var == "byte"
 	 get_var1_byte(ind)
        elsif t_var == "sint"
 	 get_var1_sint(ind)
@@ -301,14 +495,16 @@ module NumRu
        elsif t_var == "float"
 	 get_var1_float(ind)
        else
-	 raise NetcdfError,"variable type isn't supported in netCDF"
+	 raise NetcdfError,"variable type #{t_var} isn't supported in netCDF"
        end
      elsif hash.key?("start")==true
        h_sta = hash["start"]
        endq = hash.key?("end")
        strq = hash.key?("stride")
        if endq == false && strq == false
-	 if t_var == "byte"
+	 if t_var == "char"
+	   get_vars_char(h_sta,nil,nil)
+	 elsif t_var == "byte"
 	   get_vars_byte(h_sta,nil,nil)
 	 elsif t_var == "sint" 
 	   get_vars_sint(h_sta,nil,nil)
@@ -319,11 +515,13 @@ module NumRu
 	 elsif t_var == "float"
 	   get_vars_float(h_sta,nil,nil)
 	 else 
-	   raise NetcdfError "varialbe type isn't supported in netCDF"
+	   raise NetcdfError "varialbe type #{t_var} isn't supported in netCDF"
 	 end
        elsif endq == true && strq == false
 	 h_end = hash["end"]
-	 if t_var == "byte"
+ 	 if t_var == "char"
+	   get_vars_char(h_sta,h_end,nil)
+ 	 elsif t_var == "byte"
 	   get_vars_byte(h_sta,h_end,nil)
 	 elsif t_var == "sint"
 	   get_vars_sint(h_sta,h_end,nil)
@@ -334,11 +532,13 @@ module NumRu
 	 elsif t_var == "float"
 	   get_vars_float(h_sta,h_end,nil)
 	 else
-	   raise NetcdfError, "variable type isn't supported in netCDF"
+	   raise NetcdfError, "variable type #{t_var} isn't supported in netCDF"
 	 end
        elsif endq == false && strq == true
 	 h_str = hash["stride"]
-	 if t_var == "byte"
+ 	 if t_var == "char"
+	   get_vars_char(h_sta,nil,h_str)
+ 	 elsif t_var == "byte"
 	   get_vars_byte(h_sta,nil,h_str)
 	 elsif t_var == "sint"
 	   get_vars_sint(h_sta,nil,h_str)
@@ -349,12 +549,14 @@ module NumRu
 	 elsif t_var == "float"
 	   get_vars_float(h_sta,nil,h_str)
 	 else
-	   raise NetcdfError, "variable type isn't supported in netCDF"
+	   raise NetcdfError, "variable type #{t_var} isn't supported in netCDF"
 	 end
        else endq == true && strq == true
 	 h_end = hash["end"]
 	 h_str = hash["stride"]
-	 if t_var == "byte"
+	 if t_var == "char"
+	   get_vars_char(h_sta,h_end,h_str)
+	 elsif t_var == "byte"
 	   get_vars_byte(h_sta,h_end,h_str)
 	 elsif t_var == "sint"
 	   get_vars_sint(h_sta,h_end,h_str)
@@ -365,7 +567,7 @@ module NumRu
 	 elsif t_var == "float"
 	   get_vars_float(h_sta,h_end,h_str)
 	 else
-	   raise NetcdfError, "variable type isn't supported in netCDF"
+	   raise NetcdfError, "variable type #{t_var} isn't supported in netCDF"
 	 end
        end
      else
@@ -373,6 +575,8 @@ module NumRu
      end
    end
 
+   alias get simple_get
+
    def [](*a)
      first = Array.new
      last = Array.new
@@ -385,16 +589,16 @@ module NumRu
 	 stride.push(1)
        elsif(i.is_a?(Range))
 	 first.push(i.first)
-	 last.push(i.last)
+	 last.push(i.exclude_end? ? i.last-1 : i.last)
 	 stride.push(1)
        elsif(i.is_a?(Hash))
 	 r = (i.to_a[0])[0]
          s = (i.to_a[0])[1]
          if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
-	    raise "Hash argument must be {first..last, step}"
+	    raise "Hash argument must be {a_Range, step}"
 	 end
 	 first.push(r.first) 
-	 last.push(r.last)
+	 last.push(r.exclude_end? ? r.last-1 : r.last)
 	 stride.push(s)
 	 set_stride = true
        elsif(i.is_a?(TrueClass))
@@ -405,10 +609,16 @@ module NumRu
      }
 
      if(set_stride)
-	self.get({"start"=>first, "end"=>last, "stride"=>stride})
+       na = self.get({"start"=>first, "end"=>last, "stride"=>stride})
      else
-	self.get({"start"=>first, "end"=>last})
+       na = self.get({"start"=>first, "end"=>last})
      end
+     shape = na.shape
+     (a.length-1).downto(0){ |i|
+	 shape.delete_at(i) if a[i].is_a?(Fixnum)
+      }
+      na.reshape!( *shape )
+     na
    end
 
    def []=(*a)
@@ -424,7 +634,7 @@ module NumRu
 	 stride.push(1)
        elsif(i.is_a?(Range))
 	 first.push(i.first)
-	 last.push(i.last)
+	 last.push(i.exclude_end? ? i.last-1 : i.last)
 	 stride.push(1)
        elsif(i.is_a?(Hash))
 	 r = (i.to_a[0])[0]
@@ -433,7 +643,7 @@ module NumRu
 	    raise "Hash argument must be {first..last, step}"
 	 end
 	 first.push(r.first) 
-	 last.push(r.last)
+	 last.push(r.exclude_end? ? r.last-1 : r.last)
 	 stride.push(s)
 	 set_stride = true
        elsif(i.is_a?(TrueClass))
@@ -450,12 +660,8 @@ module NumRu
      end
    end
 
-   def uri
-     file.uri+'?'+name
-   end
-   
    def inspect
-     'NetCDFVar:'+uri
+     'NetCDFVar:'+file.path+'?var='+name
    end
    
  end
@@ -475,5 +681,14 @@ module NumRu
    def inspect
      'NetCDFDim:'+name
    end
+
+   def length_ul0
+      if unlimited?
+	 0
+      else
+	 length
+      end
+   end
+
  end
 end
